import { world, system, Vector, EquipmentSlot, BlockPermutation } from "@minecraft/server";
import { setTimeout } from "./timers.js";
import { portal } from "./portal/portal.js";
import { slimeSkin } from "./slimeSkin.js";
import { obeliskSpawnPoint } from "./obelisk.js"
import { chests } from "./chests.js"

system.beforeEvents.watchdogTerminate.subscribe(data => {
    data.cancel = true;
});

obeliskSpawnPoint();

system.runInterval(async () => {
    for (const player of world.getPlayers()) {
        const { x, y, z } = player.location;
        const equipment = player.getComponent("minecraft:equippable");
        const helmet = equipment?.getEquipment(EquipmentSlot.Head);
        const chestplate = equipment?.getEquipment(EquipmentSlot.Chest);
        const feet = equipment?.getEquipment(EquipmentSlot.Feet);
        const legs = equipment?.getEquipment(EquipmentSlot.Legs);
        if (helmet) {
            if (helmet.typeId == 'betterend:crystalite_helmet') {
                if (!player.getEffect('regeneration')) {
                    player.addEffect('regeneration', 100, { showParticles: false });
                }
            }
        }
        if (chestplate) {
            if (chestplate.typeId == 'betterend:crystalite_chestplate') {
                player.addEffect('haste', 100, { showParticles: false });
            }
        }
        if (feet) {
            if (feet.typeId == 'betterend:crystalite_boots') {
                player.addEffect('speed', 100, { showParticles: false });
            }
        }
        if (legs) {
            if (legs.typeId == 'betterend:crystalite_leggings') {
                player.addEffect('health_boost', 100, { showParticles: false });
            }
        }


        //FOGS, MUSIC AND SOUNDS

        const biomeTs = [
            "foggy_mushroomland", //0
            "amberland", //1
            "blossoming_spire", // 2
            'crystal_moutains', // 3
            'chorus_forest', // 4
            'dust_wasteland',  // 5 | painted mountains and oasis too
            "megalakes", // 6
            "shadow_forest", // 7
            "sulphur_springs", // 8
            "umbrella_jungle", // 9
            "glowing_grasslands", // 10
            "lantern_woods", // 11
            "dragon_graveyards", // 12
            "dry_shrubland", // 13,
            "lush_cave", // 14
            "jadestone", // 15
            "smaragdant" // 16
        ];

        if (player.dimension.id == 'minecraft:the_end') {
            const blockGround = player.dimension.getBlock(new Vector(x, y - 1, z));
            if (blockGround.typeId == 'betterend:end_moss') {
                player.addTag(`${biomeTs[0]}`);
            }
            if (blockGround.typeId == 'betterend:amber_moss') {
                player.addTag(`${biomeTs[1]}`);
            }
            if (blockGround.typeId == 'betterend:pink_moss') {
                player.addTag(`${biomeTs[2]}`);
            }
            if (blockGround.typeId == 'betterend:crystal_moss') {
                player.addTag(`${biomeTs[3]}`)
            }
            if (blockGround.typeId == 'betterend:chorus_nylium') {
                player.addTag(`${biomeTs[4]}`)
            }
            if (blockGround.typeId == 'betterend:endstone_dust' || blockGround.typeId == 'betterend:end_moss_neon_oasis') { //painted mountains and oasis too
                player.addTag(`${biomeTs[5]}`)
            }
            if (blockGround.typeId == 'betterend:end_moss_lakes') {
                player.addTag(`${biomeTs[6]}`)
            }
            if (blockGround.typeId == 'betterend:shadow_grass') {
                player.addTag(`${biomeTs[7]}`)
            }
            if (blockGround.typeId == 'betterend:brimstone' || blockGround.typeId == 'betterend:inactive_brimstone' || blockGround.typeId == 'betterend:sulphuric_rock_top') {
                player.addTag(`${biomeTs[8]}`)
            }
            if (blockGround.typeId == 'betterend:jungle_moss') {
                player.addTag(`${biomeTs[9]}`);
            }
            if (blockGround.typeId == 'betterend:end_moss_glow') {
                player.addTag(`${biomeTs[10]}`)
            }
            if (blockGround.typeId == 'betterend:sangnum') {
                player.addTag(`${biomeTs[12]}`)
            }
            if (blockGround.typeId == 'betterend:rutiscus') {
                player.addTag(`${biomeTs[13]}`)
            }
            if (blockGround.typeId == 'betterend:cave_moss') {
                player.addTag(`${biomeTs[14]}`)
            }
            if (blockGround.typeId == 'betterend:azure_jadestone' || blockGround.typeId == 'betterend:sandy_jadestone' || blockGround.typeId == 'betterend:virid_jadestone') {
                player.addTag(`${biomeTs[15]}`)
            }
            if (blockGround.typeId == 'betterend:cave_moss_smaragdant') {
                player.addTag(`${biomeTs[16]}`)
            }
            if (blockGround.typeId == 'minecraft:end_stone') {
                player.removeTag('in_biome');
                player.runCommandAsync('fog @s remove end_fog');
                for (let i = 0; i <= 16; i++) {
                    player.removeTag(biomeTs[i]);
                }
            }
        }

        /**
         * @param {Entity} player
         * @param {String} sound
         * @param {String} music
         * @param {String} fog
         * @param {Number} time
         * @example
         * fogSoundsMusic(player, 'mySound', 'myMusic', 'myFog' 1000)
         */
        async function fogSoundsMusic(player, sound, music, fog, time) {
            for (let loop = 0; loop < 1; loop++) {
                player.addTag("in_biome");
                await player.runCommandAsync(`stopsound @s`);
                await player.runCommandAsync(`fog @s remove end_fog`)
                await player.runCommandAsync(`fog @s push ${fog} end_fog`)
                setTimeout(() => {
                    player.playSound(sound, { location: player.location });
                    player.playSound(music, { location: player.location, volume: 0.3 });
                }, time);
            }
        }

        //foggy
        if (player.hasTag(`${biomeTs[0]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'foggy_mushroomland', 'betterend.calm.music', 'betterend:foggy', 1000);
        }
        //amber
        if (player.hasTag(`${biomeTs[1]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'amber_land', 'betterend.calm.music', 'betterend:amberland', 1000);
        }
        //blossoming
        if (player.hasTag(`${biomeTs[2]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'blossoming_spires', 'betterend.calm.music', 'betterend:blossoming', 1000);
        }
        //crystal mountains
        if (player.hasTag(`${biomeTs[3]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'foggy_mushroomland', 'betterend.caves.music', 'betterend:crystal', 1000);
        }
        //chorus forest
        if (player.hasTag(`${biomeTs[4]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'chorus_forest', 'betterend.forest.music', 'betterend:chorus_forest', 1000);
        }
        //dust & painted mountains & oasis
        if (player.hasTag(`${biomeTs[5]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'dust_wastelands', 'betterend.dark.music', 'betterend:dust', 1000);
        }
        //megalakes
        if (player.hasTag(`${biomeTs[6]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'megalake', 'betterend.calm.music', 'betterend:megalakes', 1000);
        }
        //shadow forest
        if (player.hasTag(`${biomeTs[7]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'chorus_forest', 'betterend.dark.music', 'betterend:shadow', 1000);
        }
        //sulphur springs
        if (player.hasTag(`${biomeTs[8]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'sulphur_springs', 'betterend.caves.music', 'betterend:sulphur', 1000);
        }
        //umbrella jungle
        if (player.hasTag(`${biomeTs[9]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'umbrella_jungle', 'betterend.calm.music', 'betterend:umbrella', 1000);
        }
        //glowing grasslands
        if (player.hasTag(`${biomeTs[10]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'glowing_grasslands', 'betterend.calm.music', 'betterend:glowing_grasslands', 1000);
        }
        //lantern woods
        if (player.hasTag(`${biomeTs[11]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'blossoming_spires', 'betterend.calm.music', 'betterend:lantern', 1000);
        }
        //dragon graveyards
        if (player.hasTag(`${biomeTs[12]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'amber_land', 'betterend.calm.music', 'betterend:dragon_graveyards', 1000);
        }
        //dry shrubland
        if (player.hasTag(`${biomeTs[13]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'chorus_forest', 'betterend.forest.music', 'betterend:dry', 1000);
        }
        //lush cave
        if (player.hasTag(`${biomeTs[14]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'caves', 'betterend.caves.music', 'betterend:lush_cave', 1000);
        }
        //jadestone
        if (player.hasTag(`${biomeTs[15]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'caves', 'betterend.caves.music', 'betterend:jadestone', 1000);
        }
        //smaragdant
        if (player.hasTag(`${biomeTs[16]}`) && !player.hasTag("in_biome")) {
            fogSoundsMusic(player, 'caves', 'betterend.caves.music', 'betterend:smaragdant', 1000);
        }

        const matchingTags = biomeTs.filter(tag => player.hasTag(tag));
        if (matchingTags.length >= 2) {
            player.removeTag('in_biome');
            for (let i = 0; i <= 16; i++) {
                player.removeTag(biomeTs[i]);
            }
        }
    }
});

system.runInterval(async () => {

    for (const entities of world.getDimension('overworld').getEntities()) {
        //chests
        chests(entities, Vector, BlockPermutation);
        //slimesSkin
        slimeSkin(entities, Vector);

        //portal crystals
        if (entities.typeId == 'betterend:check_portal') {
            const { successCount: crystals } = await entities.runCommandAsync(`testfor @e[type=betterend:eternal_crystal_entity,r=7,c=6]`);
            if (crystals == 6) {
                entities.runCommandAsync(`tag @p[r=2,tag=!teleport_to_overworld] add teleport_to_overworld`);
                if (!entities.hasTag('portal_put')) {
                    entities.addTag('portal_put');
                    await entities.runCommandAsync(`structure load open_portal ~ ~ ~-2`);
                }
            }
            else if (crystals < 6 && entities.hasTag('portal_put')) {
                entities.removeTag('portal_put');
                await entities.runCommandAsync(`structure load close_portal ~ ~ ~-2`);
            }
        }
        if (entities.typeId == 'betterend:check_portal_overworld') {
            entities.runCommandAsync(`tag @p[r=2,tag=!teleport_to_end] add teleport_to_end`);
        }
    }
    for (const player of world.getPlayers()) {

        //obelisk

        if (player.hasTag('waiting_obelisk')) {
            player.removeTag('waiting_obelisk');
            const playerSpawns = player.getTags();
            const endSpawnTag = playerSpawns.find(tag => tag.includes("endspawn"));
            const coords = endSpawnTag.split(':')[2].split(' ');
            setTimeout(() => {
                player.teleport(new Vector(parseInt(coords[0]), parseInt(coords[1]), parseInt(coords[2])), { dimension: world.getDimension("the end") });
                player.removeTag(endSpawnTag);
            }, 5000);
        }

        //portal

        portal();

        //equipment
    }
});

world.beforeEvents.itemUseOn.subscribe(async evd => {
    const { source: player, itemStack: item, block } = evd;

    if (item.typeId == 'betterend:eternal_crystal' && block.typeId == 'betterend:eternal_pedestal_off' && !player.isSneaking) {
        try {
            system.run(async () => {
                await player.runCommandAsync(`clear @s betterend:eternal_crystal 0 1`);
                await player.playSound('beacon.power', { location: player.location, volume: 1 })
            });
        } catch (e) { console.warn(e) }
    }
});

world.afterEvents.playerPlaceBlock.subscribe(evd => {
    const { block, player } = evd;
    const { x, y, z } = block.location;
    const chestsID = ["umbrella_tree_chest", "tenanea_chest", "pythadendron_chest", "lucernia_chest", "lacugrove_chest", "jellyshroom_chest", "helix_tree_chest", "end_lotus_chest", "dragon_tree_chest"];
    for (const chest of chestsID) {
        if (block.typeId.includes(chest)) {
            const chest = player.dimension.spawnEntity('betterend:chest_test', new Vector(x, y, z));
            setTimeout(() => {
                chest.teleport(new Vector(x + 0.5, y, z + 0.5));
            }, 1000);
        }
    }
});